/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.openrdf.elmo.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import javax.xml.namespace.QName;
import org.openide.util.Lookup;
import org.openide.util.LookupListener;
import org.openrdf.elmo.Entity;
import it.tidalwave.openrdf.elmo.ElmoManagerProxy;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: EntityBasedLookup.java,v 4fbb06e9364c 2011/03/06 21:25:16 fabrizio $
 *
 ******************************************************************************/
public abstract class EntityBasedLookup<T extends Entity> extends Lookup
  {
    private T entity;

    @CheckForNull
    private QName qName;

    private static final ElmoManagerProxy em = new ElmoManagerProxy();

    /***************************************************************************
     *
     *
     **************************************************************************/
    public EntityBasedLookup (@Nonnull final T entity)
      {
        // We can't call entity.getQName() here - entity might be not initialized
        this.entity = entity;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    @Nonnull
    public <T> Result<T> lookup (@Nonnull final Template<T> template)
      {
        return new Result<T>()
          {
            @Override
            public void addLookupListener (LookupListener arg0)
              {
              }

            @Override
            public void removeLookupListener (LookupListener arg0)
              {
              }

            @Override
            public Collection<? extends T> allInstances()
              {
                return Collections.singletonList(lookup(template.getType()));
              }
          };
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    protected T getEntity()
      {
        synchronized (this)
          {
            // Now we can get rid of the entity, and we keep the name
            if (qName == null)
              {
                qName = em.merge(entity).getQName();
                entity = null;
              }
          }

        return (T)em.find(qName);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    protected synchronized QName getQName()
      {
        if (qName == null)
          {
            qName = getEntity().getQName();
          }

        return qName;
      }
  }
