/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.openrdf.elmo.impl;

import javax.xml.namespace.QName;
import org.openrdf.elmo.Entity;
import it.tidalwave.util.Id;
import javax.annotation.Nonnull;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: IdLazyLookup.java,v 4fbb06e9364c 2011/03/06 21:25:16 fabrizio $
 *
 ******************************************************************************/
public class IdLazyLookup extends EntityBasedLookup<Entity>
  {
    private Id id;

    /***************************************************************************
     *
     *
     **************************************************************************/
    public IdLazyLookup (@Nonnull final Entity entity)
      {
        super(entity);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    public synchronized <T> T lookup (Class<T> clazz)
      {
        if (!Id.class.equals(clazz))
          {
            return null;
          }

        if (id == null)
          {
            final QName qName = getQName();
            id = new Id(qName.getNamespaceURI() + qName.getLocalPart());
          }
        
        return (T)id;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    @Nonnull
    public String toString()
      {
        return String.format("IdLazyLookup@%x", System.identityHashCode(this));
      }
  }
