/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.openrdf.elmo.impl;

import it.tidalwave.util.logging.Logger;
import it.tidalwave.semantic.Wrapper;
import java.util.Collection;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import org.openide.util.Lookup.Result;
import org.openide.util.LookupListener;
import org.openide.util.NotImplementedException;


class WrappedResult<T> extends Result<T>
  {
    @Nonnull
    private final Result<T> delegate;

    public WrappedResult (@Nonnull final Result<T> delegate)
      {
        this.delegate = delegate;
      }

    @Override
    public void addLookupListener(LookupListener arg0)
      {
        throw new NotImplementedException();
//        delegate.addLookupListener(arg0);
      }

    @Override
    public void removeLookupListener(LookupListener arg0)
      {
        throw new NotImplementedException();
//        delegate.removeLookupListener(arg0);
      }

    @Override
    public Collection<? extends T> allInstances()
      {
        return Wrapper.wrap(delegate.allInstances());
      }
  }

/**
 *
 * @author fritz
 */
public class WrappedLookup extends Lookup
  {
    private static final String CLASS = WrappedLookup.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private final Object owner;

    @Nonnull
    private final Lookup delegate;

    public WrappedLookup (@Nonnull final Object owner, @Nonnull final Lookup delegate)
      {
        this.owner = owner;
        this.delegate = delegate;
      }

    @Override
    public <T> T lookup (@Nonnull final Class<T> clazz)
      {
        logger.finest("lookup(%s)", clazz);
        // FIXME: handle QName: ((Entity)owner).getQName();
        if (clazz.isAssignableFrom(owner.getClass()))
          {
            logger.finest(">>>> returning owner: %s", owner);
            return (T)owner;
          }
        
        T object = delegate.lookup(clazz);
        logger.finest(">>>> returning %s from the delegate", object);
        //
        // For instance, you're doing lookup(Concept.class): but it could anyway
        // return an instance of a proxied class; so wrapping depends on what
        // the caller is explicitly asking for.
        //
        if (Wrapper.isRegistered(clazz))
          {
            object = Wrapper.wrap(object);
          }
        
        // FIXME: would be better to cache wrapped items?
        return object;
      }

    @Override
    public <T> Result<T> lookup (@Nonnull final Template<T> clazz)
      {
        return new WrappedResult<T>(delegate.lookup(clazz));
      }

    @Override
    public String toString()
      {
        return String.format("WrappedLookup[%s]", delegate.toString());
      }
  }
