/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import it.tidalwave.openrdf.elmo.impl.ElmoWrapper;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id: Wrapper.java,v 4fbb06e9364c 2011/03/06 21:25:16 fabrizio $
 *
 **********************************************************************************************************************/
public class Wrapper
  {
    /*******************************************************************************************************************
     *
     * A factory for creating entity proxies.
     *
     ******************************************************************************************************************/
    public static interface WrapperSpi
      {
        @CheckForNull
        public<X> X wrap (@Nullable  X object);

        @CheckForNull
        public<X> X unwrap (@CheckForNull  X object);

        public boolean isRegistered (@Nonnull Class<?> clazz);

        public boolean isWrapped (@Nullable Object object);

        public void register (@Nonnull Class<?> interfaceClass,
                              @Nonnull Class<?> implementationClass);

        @CheckForNull
        public <T> Class<? extends T> findImplementation (@Nonnull final Class<T> clazz);
      }

    private final static WrapperSpi spi = new ElmoWrapper(); // TOOD: use a Lookup

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private Wrapper()
      {
      }

    /*******************************************************************************************************************
     *
     * Creates a proxy for the given entity. If a non-entity is passed as parameter, it is returned as is. If an object
     * that has been already wrapped is passed as parameter, it is returned as is. {@code null} is accepted as a
     * parameter, and in this case {@code null} is returned.
     *
     * @param  proxy  the object
     * @return        the proxy
     *
     ******************************************************************************************************************/
    @CheckForNull
    public static <X> X wrap (final @Nullable X object)
      {
        return spi.wrap(object);
      }

    /*******************************************************************************************************************
     *
     * Extracts a wrapped entity from a proxy. If a non-proxy is passed as parameter, it is returned as is. {@code null}
     * is accepted as a parameter, and in this case {@code null} is returned.
     *
     * @param  proxy  the proxy
     * @return        the proxied object
     *
     ******************************************************************************************************************/
    @CheckForNull
    public static <X> X unwrap (final @Nullable X proxy)
      {
        return spi.unwrap(proxy);
      }

    /*******************************************************************************************************************
     *
     * Returns {@code true} if the passed object is a proxy.
     *
     * @param   object   the probed object
     * @return           {@code true} if it's a proxy
     *
     ******************************************************************************************************************/
    public static boolean isWrapped (@Nullable Object object)
      {
        return spi.isWrapped(object);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static void register (@Nonnull final Class<?> interfaceClass,
                                 @Nonnull final Class<?> implementationClass)
      {
        spi.register(interfaceClass, implementationClass);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static boolean isRegistered (@Nonnull final Class<?> clazz)
      {
        return spi.isRegistered(clazz);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @CheckForNull
    public static <T> Class<? extends T> findImplementation (@Nonnull final Class<T> clazz)
      {
        return spi.findImplementation(clazz);
      }
  }
